///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoAttrComp.cc
// ---------------
// Cego database table attribute comparison description
//      
// Design and Implementation by Bjoern Lemke
//     
// (C)opyright 2000-2017 Bjoern Lemke
//
// IMPLEMENTATION MODULE
//
// Class: CegoAttrComp
//
// Description: Attribute comparison utility class
//
// Status: CLEAN
//
///////////////////////////////////////////////////////////////////////////////

// CEGO INCLUDES
#include "CegoAttrComp.h"

// POSIX INCLUDES
#include <stdlib.h>
#include <string.h>

CegoAttrComp::CegoAttrComp()
{
    _compMode = UNDEF;
    _pMatcher = 0;
}

CegoAttrComp::CegoAttrComp(const Chain& tableName, const Chain& attrName)
{
    _tableName = tableName;
    _attrName = attrName;
    _compMode = UNDEF;
    _pMatcher = 0;
}

CegoAttrComp::CegoAttrComp(const Chain& tableName, const Chain& attrName, CegoComparison comp, const CegoFieldValue& fv)
{
    _tableName = tableName;
    _attrName = attrName;
    _comp = comp;
    _fv = fv;
    _compMode = VAL;
    _isSetup = false;
    _pMatcher = 0;
}

CegoAttrComp::CegoAttrComp(const Chain& tableName, const Chain& attrName, CegoComparison comp, const CegoAttrDesc& attrDesc)
{
    _tableName = tableName;
    _attrName = attrName;
    _comp = comp;
    _attrDesc = attrDesc;
    _compMode = ATTR;
    _pMatcher = 0;
    _isSetup = false;
}

CegoAttrComp::CegoAttrComp(const Chain& tableName, const Chain& attrName, const CegoFieldValue& fv, const CegoFieldValue& fv2)
{
    _tableName = tableName;
    _attrName = attrName;
    _fv = fv;
    _fv2 = fv2;
    _compMode = BTWN;
    _btwnMode = VALUE2VALUE;
    _pMatcher = 0;
    _isSetup = true;
    _isSetup2 = true;	    
}

CegoAttrComp::CegoAttrComp(const Chain& tableName, const Chain& attrName, const CegoAttrDesc& attrDesc, const CegoFieldValue& fv2)
{
    _tableName = tableName;
    _attrName = attrName;
    _attrDesc = attrDesc;
    _fv2 = fv2;
    _compMode = BTWN;
    _btwnMode = ATTR2VALUE;
    _pMatcher = 0;
    _isSetup = false;
    _isSetup2 = true;
}

CegoAttrComp::CegoAttrComp(const Chain& tableName, const Chain& attrName, const CegoFieldValue& fv, const CegoAttrDesc& attrDesc2)
{
    _tableName = tableName;
    _attrName = attrName;
    _fv = fv;
    _attrDesc2 = attrDesc2;
    _compMode = BTWN;
    _btwnMode = VALUE2ATTR;
    _pMatcher = 0;
    _isSetup = true;
    _isSetup2 = false;
}

CegoAttrComp::CegoAttrComp(const Chain& tableName, const Chain& attrName, const CegoAttrDesc& attrDesc, const CegoAttrDesc& attrDesc2)
{
    _tableName = tableName;
    _attrName = attrName;
    _attrDesc = attrDesc;
    _attrDesc2 = attrDesc2;
    _compMode = BTWN;
    _btwnMode = ATTR2ATTR;
    _pMatcher = 0;
    _isSetup = false;
    _isSetup2 = false;
}

CegoAttrComp::CegoAttrComp(const Chain& tableName, const Chain& attrName, const Chain& pattern, bool isNot)
{
    _tableName = tableName;
    _attrName = attrName;
    _pattern = pattern;
    
    if ( isNot )
	_compMode = ISNOTLIKE;
    else
	_compMode = ISLIKE;
    _pMatcher = 0;
}

CegoAttrComp::~CegoAttrComp()
{
    if ( _pMatcher )
	delete _pMatcher;
}

CegoAttrComp::CompMode CegoAttrComp::getCompMode() const
{
    return _compMode;
}

const Chain& CegoAttrComp::getTableName() const
{
    return _tableName;
}

const Chain& CegoAttrComp::getAttrName() const
{
    return _attrName;
}

CegoComparison CegoAttrComp::getComparison() const
{
    return _comp;
}

void CegoAttrComp::setComparison(CegoComparison comp)
{
    _comp = comp;
}

void CegoAttrComp::setFieldValue(const CegoFieldValue& fv)
{
    _fv = fv;
}

void CegoAttrComp::setFieldValue2(const CegoFieldValue& fv)
{
    _fv2 = fv;
}

const CegoFieldValue& CegoAttrComp::getFieldValue() const
{
    return _fv;
}

const CegoFieldValue& CegoAttrComp::getFieldValue2() const
{
    return _fv2;
}

const CegoAttrDesc& CegoAttrComp::getAttrDesc() const
{
    return _attrDesc;
}

bool CegoAttrComp::isSetup()
{
    if ( _compMode == ATTR )
	return _isSetup;
    if ( _compMode == BTWN )
	return _isSetup && _isSetup2;
    // in any other case, not relevant
    return true;
}

const Chain& CegoAttrComp::getPattern() const
{
    return _pattern;
}

Matcher* CegoAttrComp::getMatcher()
{
    if ( _pMatcher == 0)
    {	
	Chain sqlPattern = Chain("^") + _pattern + Chain("$");
	sqlPattern.replaceAll(Chain("%"), Chain(".*"), sqlPattern);
	sqlPattern.replaceAll(Chain("_"), Chain("."), sqlPattern);	
	_pMatcher = new Matcher(sqlPattern);
	_pMatcher->prepare();
	
    }
    return _pMatcher;
}

void CegoAttrComp::reset()
{
    if ( _compMode == ATTR )
	_isSetup=false;
    else if ( _compMode == BTWN )
    {
	if ( _btwnMode == VALUE2VALUE )
	{
	    _isSetup=true;
	    _isSetup2=true;
	}
	else if ( _btwnMode == VALUE2ATTR )
	{
	    _isSetup=true;
	    _isSetup2=false;
	}
	else if ( _btwnMode == ATTR2VALUE )
	{
	    _isSetup=false;
	    _isSetup2=true;
	}
	else if ( _btwnMode == ATTR2ATTR )
	{
	    _isSetup=false;
	    _isSetup2=false;
	}
    }
}

bool CegoAttrComp::setup(const ListT<CegoField>& fl)
{
    // _isSetup=false;

    if ( _compMode == ATTR )
    {
	CegoField* pF = fl.First();
	while ( pF && _isSetup == false )
	{	
	    if ( ( _attrDesc.getTableName() == pF->getTableName() || _attrDesc.getTableName()== pF->getTableAlias() )
		 && _attrDesc.getAttrName() == pF->getAttrName() )
	    {
		_isSetup=true;
		_fv = pF->getValue();
	    }
	    pF = fl.Next();
	}
	return _isSetup;
    }
    else if ( _compMode == BTWN )
    {
	CegoField* pF = fl.First();
	while ( pF && _isSetup == false )
	{	
	    if ( ( _attrDesc.getTableName() == pF->getTableName() || _attrDesc.getTableName()== pF->getTableAlias() )
		 && _attrDesc.getAttrName() == pF->getAttrName() )
	    {
		_isSetup=true;
		_fv = pF->getValue();
	    }
	    pF = fl.Next();
	}
	pF = fl.First();
	while ( pF && _isSetup2 == false )
	{	
	    if ( ( _attrDesc2.getTableName() == pF->getTableName() || _attrDesc2.getTableName()== pF->getTableAlias() )
		 && _attrDesc2.getAttrName() == pF->getAttrName() )
	    {
		_isSetup2=true;
		_fv2 = pF->getValue();
	    }
	    pF = fl.Next();
	}
	return _isSetup && _isSetup2;
    }
    else
    {
	throw Exception(EXLOC, Chain("Cannot setup attribute comparison"));
    }
}

bool CegoAttrComp::setup(ListT<CegoField>** pJoinBuf, int offset)
{

    int i = offset;

    if ( _compMode == ATTR )
    {

	while ( pJoinBuf[i] && _isSetup == false )
	{
	    CegoField* pF = pJoinBuf[i]->First();
	    while (pF && _isSetup == false)
	    {	
		// cout << "Checking T=" << pF->getTableName() << " A=" << pF->getAttrName() << "Alias=" << pF->getTableAlias() << endl;
		if ( ( _attrDesc.getTableName() == pF->getTableName() || _attrDesc.getTableName()== pF->getTableAlias() )
		     && _attrDesc.getAttrName() == pF->getAttrName() )
		{
		    _isSetup = true;		    
		    _fv = pF->getValue();
		}
		pF = pJoinBuf[i]->Next();
	    }
	    i++;
	}
	return _isSetup;
    }
    else if ( _compMode == BTWN )
    {	
	while ( pJoinBuf[i] && ( _isSetup == false || _isSetup2 == false ) )
	{
	    CegoField* pF = pJoinBuf[i]->First();
	    while (pF && _isSetup == false)
	    {	
		// cout << "1 Checking " << _attrDesc.getAttrName() T=" << pF->getTableName() << " A=" << pF->getAttrName() << " Alias=" << pF->getTableAlias() << endl;
		if ( ( _attrDesc.getTableName() == pF->getTableName() || _attrDesc.getTableName()== pF->getTableAlias() )
		     && _attrDesc.getAttrName() == pF->getAttrName() )
		{		    
		    _isSetup = true;		    
		    _fv = pF->getValue();
		}
		pF = pJoinBuf[i]->Next();
	    }
	    pF = pJoinBuf[i]->First();
	    while (pF && _isSetup2 == false)
	    {	
		// cout << "2 Checking T=" << pF->getTableName() << " A=" << pF->getAttrName() << "Alias=" << pF->getTableAlias() << endl;
		if ( ( _attrDesc2.getTableName() == pF->getTableName() || _attrDesc2.getTableName()== pF->getTableAlias() )
		     && _attrDesc2.getAttrName() == pF->getAttrName() )
		{
		    _isSetup2 = true;		    
		    _fv2 = pF->getValue();
		}
		pF = pJoinBuf[i]->Next();
	    }
	    i++;
	}

	return _isSetup && _isSetup2;
    }
    else
    {
	throw Exception(EXLOC, Chain("Cannot setup attribute comparison"));
    }
}

CegoAttrComp& CegoAttrComp::operator = ( const CegoAttrComp& ac)
{
    _tableName = ac._tableName;
    _attrName = ac._attrName;
    _comp = ac._comp;
    _fv = ac._fv;
    _fv2 = ac._fv2;
    _attrDesc = ac._attrDesc;
    _attrDesc2 = ac._attrDesc2;
    _compMode = ac._compMode;
    _btwnMode = ac._btwnMode;
    _isSetup = ac._isSetup;
    _isSetup2 = ac._isSetup2;
    _pattern = ac._pattern;
    _pMatcher = 0;
    return (*this);
}
    
bool CegoAttrComp::operator == ( const CegoAttrComp& ac) const
{
    if ( _compMode != ac._compMode )
	return false;    
    if ( _compMode == ATTR )
	return ( _attrName == ac._attrName && _comp == ac._comp );
    if ( _compMode == VAL )
	// we don't compare the values ( _fv == ac._fv ), since CegoAttrCond::update sets it up 
	return ( _attrName == ac._attrName && _comp == ac._comp ); 
    if ( _compMode == BTWN )
    {
	// we don't compare the values ( _fv == ac._fv && _fv2 == ac._fv2 ), since CegoAttrCond::update sets it up
	
	switch ( _btwnMode )
	{
	case VALUE2VALUE:
	    return ( _attrName == ac._attrName );
	case VALUE2ATTR:
	    return ( ( _attrName == ac._attrName ) && ( _attrDesc2 == ac._attrDesc2 ) );
	case ATTR2VALUE:
	    return ( ( _attrName == ac._attrName ) && ( _attrDesc == ac._attrDesc ) );
	case ATTR2ATTR:
	    return ( ( _attrName == ac._attrName )  && ( _attrDesc == ac._attrDesc ) && ( _attrDesc2 == ac._attrDesc2 ) );
	}
    }
    if ( _compMode == ISLIKE || _compMode == ISNOTLIKE )
	return ( _attrName == ac._attrName && _pattern == ac._pattern  );

    return false;
}

Chain CegoAttrComp::getId() const
{
    Chain s;
    
    s = _attrName;
    
    if ( _compMode == BTWN )
    {	
	switch ( _btwnMode )
	{
	case VALUE2VALUE:
	    s += Chain("#btw#") + _fv.toChain() + Chain("#") + _fv2.toChain();
	    break;
	case VALUE2ATTR:
	    s += Chain("#btw#") + _fv.toChain() + Chain("#") + _attrDesc2.toChain();
	    break;
	case ATTR2VALUE:
	    s += Chain("#btw#") + _attrDesc.toChain() + Chain("#") + _fv2.toChain();
	    break;	    
	case ATTR2ATTR:
	    s += Chain("#btw#") + _attrDesc.toChain() + Chain("#") + _attrDesc2.toChain();
	    break;	    	    
	}
    }
    else if ( _compMode == ISLIKE )
    {
	s += Chain("lk'") + _pattern + Chain("'"); 
    }
    else if ( _compMode == ISNOTLIKE )
    {
	s += Chain("nlk'") + _pattern + Chain("'"); 
    }
    else
    {
	switch ( _comp )
	{
	case EQUAL:
	    s += Chain("=");
	    break;
	case NOT_EQUAL:
	    s += Chain("!=");
	    break;
	case LESS_THAN:
	    s += Chain("<");
	    break;
	case MORE_THAN:
	    s += Chain(">");
	    break;
	case LESS_EQUAL_THAN:
	    s += Chain("<=");
	    break;
	case MORE_EQUAL_THAN:
	    s += Chain(">=");
	    break;
	}
	if ( _compMode == VAL )
	    s += _fv.toChain();
	else
	{
	    s += _attrDesc.toChain();
	    if ( _isSetup )
		s += _fv.toChain();
	    else
		s += Chain("#not setup#");
	}
    }
    return s;    
}

Chain CegoAttrComp::toChain() const
{
    Chain s;
    
    s = _tableName + "." + _attrName;
    
    if ( _compMode == BTWN )
    {
	switch ( _btwnMode )
	{
	case VALUE2VALUE:
	    s += Chain(" between ") + _fv.toChain() + Chain(" and ") + _fv2.toChain();
	    break;
	case VALUE2ATTR:
	    s += Chain(" between ") + _fv.toChain() + Chain(" and ") + _attrDesc2.toChain();
	    break;
	case ATTR2VALUE:
	    s += Chain(" between ") + _attrDesc.toChain() + Chain(" and ") + _fv2.toChain();
	    break;	    
	case ATTR2ATTR:
	    s += Chain(" between ") + _attrDesc.toChain() + Chain(" and ") + _attrDesc2.toChain();
	    break;	    	    
	}
    }
    else if ( _compMode == ISLIKE )
    {
	s += Chain(" like '") + _pattern + Chain("'"); 
    }
    else if ( _compMode == ISNOTLIKE )
    {
	s += Chain(" not like '") + _pattern + Chain("'"); 
    }
    else
    {
	switch ( _comp )
	{
	case EQUAL:
	    s += Chain(" = ");
	    break;
	case NOT_EQUAL:
	    s += Chain(" != ");
	    break;
	case LESS_THAN:
	    s += Chain(" < ");
	    break;
	case MORE_THAN:
	    s += Chain(" > ");
	    break;
	case LESS_EQUAL_THAN:
	    s += Chain(" <= ");
	    break;
	case MORE_EQUAL_THAN:
	    s += Chain(" >= ");
	    break;
	}
	if ( _compMode == VAL )
	    s += _fv.toChain();
	else
	{
	    s += _attrDesc.toChain();
	    if ( _isSetup )
		s += Chain("(") + _fv.toChain() + Chain(")");
	    else
		s += Chain(" -> not setup ");
	}
    }
    return s;
}

ostream& operator << (ostream& s, const CegoAttrComp& ac)
{
    s << ac.toChain();
    return s;
}
